from abc import abstractmethod

from checkov.common.checks.base_check import BaseCheck
from checkov.common.multi_signature import multi_signature
from checkov.terraform.checks.data.registry import data_registry


class BaseDataCheck(BaseCheck):
    def __init__(self, name, id, categories, supported_data):
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_data,
                         block_type="data")
        self.supported_data = supported_data
        data_registry.register(self)

    def scan_entity_conf(self, conf, entity_type):
        return self.scan_data_conf(conf, entity_type)

    @abstractmethod
    @multi_signature()
    def scan_data_conf(self, conf, entity_type):
        raise NotImplementedError()

    @classmethod
    @scan_data_conf.add_signature(args=["self", "conf"])
    def _scan_data_conf_self_conf(cls, wrapped):
        def wrapper(self, conf, entity_type=None):
            # keep default argument for entity_type so old code, that doesn't set it, will work.
            return wrapped(self, conf)

        return wrapper



from abc import abstractmethod

from checkov.common.checks.base_check import BaseCheck
from checkov.terraform.checks.provider.registry import provider_registry


class BaseProviderCheck(BaseCheck):
    def __init__(self, name, id, categories, supported_provider):
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_provider,
                         block_type="provider")
        self.supported_provider = supported_provider
        provider_registry.register(self)

    def scan_entity_conf(self, conf, entity_type):
        # entity_type is always 'provider'
        return self.scan_provider_conf(conf)

    @abstractmethod
    def scan_provider_conf(self, conf):
        raise NotImplementedError()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck
#from checkov.arm.base_resource_value_check import BaseResourceCheck
import re

'''
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
import re
'''

# https://docs.microsoft.com/en-us/azure/templates/microsoft.network/networksecuritygroups
# https://docs.microsoft.com/en-us/azure/templates/microsoft.network/networksecuritygroups/securityrules

INTERNET_ADDRESSES = ["*", "0.0.0.0", "<nw>/0", "/0", "internet", "any"]
PORT_RANGE = re.compile('\d+-\d+')

class NSGRulePortAccessRestricted(BaseResourceCheck):
    def __init__(self, name, check_id, port):
        supported_resources = ['Microsoft.Network/networkSecurityGroups',
                               'Microsoft.Network/networkSecurityGroups/securityRules']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def is_port_in_range(self, portRange):
        if re.match(PORT_RANGE, portRange):
            start, end = int(portRange.split('-')[0]), int(portRange.split('-')[1])
            if start <= self.port <= end:
                return True
        if portRange in [str(self.port), '*']:
            return True
        return False

    def scan_resource_conf(self, conf):
        if "properties" in conf:
            securityRules = []
            if "type" in conf and conf["type"] == "Microsoft.Network/networkSecurityGroups":
                if "securityRules" in conf["properties"]:
                    securityRules.extend(conf["properties"]["securityRules"])
            if "type" in conf and conf["type"] == "Microsoft.Network/networkSecurityGroups/securityRules":
                securityRules.append(conf)

            for rule in securityRules:
                portRanges = []
                sourcePrefixes = []
                if "properties" in rule:
                    if "access" in rule["properties"] and rule["properties"]["access"].lower() == "allow":
                        if "direction" in rule["properties"] and rule["properties"]["direction"].lower() == "inbound":
                            if "protocol" in rule["properties"] and rule["properties"]["protocol"].lower() == "tcp":
                                if "destinationPortRanges" in rule["properties"]:
                                    portRanges.extend(rule["properties"]["destinationPortRanges"])
                                if "destinationPortRange" in rule["properties"]:
                                    portRanges.append(rule["properties"]["destinationPortRange"])

                                if "sourceAddressPrefixes" in rule["properties"]:
                                    sourcePrefixes.extend(rule["properties"]["sourceAddressPrefixes"])
                                if "sourceAddressPrefix" in rule["properties"]:
                                    sourcePrefixes.append(rule["properties"]["sourceAddressPrefix"])

                                for portRange in portRanges:
                                    if self.is_port_in_range(portRange):
                                        for prefix in sourcePrefixes:
                                            if prefix in INTERNET_ADDRESSES:
                                                return CheckResult.FAILED

        return CheckResult.PASSED




from checkov.cloudformation.parser.node import dict_node, list_node, str_node
from checkov.serverless.parsers.parser import FUNCTIONS_TOKEN, PROVIDER_TOKEN, IAM_ROLE_STATEMENTS_TOKEN, \
    ENVIRONMENT_TOKEN
from checkov.cloudformation.context_parser import ContextParser as CfnContextParser


class ContextParser(object):
    """
    serverless functions template context parser
    """
    # control on inherited provider attributes to scanned functions
    ENRICHED_ATTRIBUTES = [IAM_ROLE_STATEMENTS_TOKEN, ENVIRONMENT_TOKEN]

    def __init__(self, sls_file, sls_template, sls_template_lines):
        self.sls_file = sls_file
        self.sls_template = sls_template
        self.sls_template_lines = sls_template_lines
        self.provider_conf = sls_template[PROVIDER_TOKEN]
        self.functions_conf = sls_template[FUNCTIONS_TOKEN]
        self.provider_type = self._infer_provider_type()

    def extract_function_code_lines(self, sls_function):
        find_lines_result_list = list(CfnContextParser.find_lines(sls_function, '__startline__'))
        if len(find_lines_result_list) >= 1:
            start_line = min(find_lines_result_list) - 1
            end_line = max(list(CfnContextParser.find_lines(sls_function, '__endline__')))

            entity_lines_range = [start_line, end_line - 1]

            entity_code_lines = self.sls_template_lines[start_line - 1: end_line - 1]
            return entity_lines_range, entity_code_lines
        return None, None

    def enrich_function_with_provider(self, sls_function_name):
        """
        Update inplace a function's template with pre-defined inherited provider attributes
        :param sls_function_name: scanned function
        :return: None
        """
        if isinstance(self.provider_conf, dict_node):
            for enriched_attribute in self.ENRICHED_ATTRIBUTES:
                if self.provider_conf.get(enriched_attribute):
                    provider_attribute = self.provider_conf.get(enriched_attribute)
                    template_function = self.functions_conf[sls_function_name]
                    if template_function.get(enriched_attribute):
                        if isinstance(template_function[enriched_attribute], list_node):
                            template_function[enriched_attribute].extend(provider_attribute)
                        if isinstance(template_function[enriched_attribute], dict_node):
                            template_function[enriched_attribute].update(provider_attribute)
                    else:
                        template_function[enriched_attribute] = provider_attribute

    def _infer_provider_type(self):
        if isinstance(self.provider_conf, dict_node):
            return self.provider_conf.get('name')
        if isinstance(self.provider_conf, str_node):
            return self.provider_conf



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class APIGatewayAccessLogging(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure API Gateway has Access Logging enabled"
        id = "CKV_AWS_76"
        supported_resources = ['aws_api_gateway_stage']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "access_log_settings/[0]/destination_arn"

    def get_expected_value(self):
        return ANY_VALUE


check = APIGatewayAccessLogging()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_int
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class NetworkWatcherFlowLogPeriod(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that Network Security Group Flow Log retention period is 'greater than 90 days'"
        id = "CKV_AZURE_12"
        supported_resources = ['azurerm_network_watcher_flow_log']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'enabled' in conf and conf['enabled'][0]:
            retention_block = conf['retention_policy'][0]
            if retention_block['enabled'][0]:
                retention_in_days = force_int(retention_block['days'][0])
                if retention_in_days and retention_in_days >= 90:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = NetworkWatcherFlowLogPeriod()
